package org.china.vo.das.engine;

import de.fb12.gdt.GridService;
import de.fb12.gdt.GridAttribute;
import de.fb12.gdt.GridMethod;

import net.ivoa.adql.SelectStatement;

import org.china.vo.das.parser.ADQLParser;
import org.china.vo.das.service.ExecPlan;
import org.china.vo.das.datanode.DataResourceMap;
import org.china.vo.das.exceptions.ADQLErrorException;
import org.china.vo.das.exceptions.CheckDNTableException;
import org.china.vo.das.exceptions.DataNodeNotFoundException;
import org.china.vo.das.exceptions.GetTableTypeException;
import org.china.vo.das.exceptions.ImageTableMoreThanOneErrorException;
import org.china.vo.das.exceptions.JoinedTableMoreThanOneErrorException;
import org.china.vo.das.exceptions.ParserExpressionException;
import org.china.vo.das.exceptions.ParserSyntaxException;
import org.china.vo.das.exceptions.SpecTableMoreThanOneErrorException;
import org.china.vo.das.exceptions.TableNotFoundErrorException;
import org.china.vo.das.exceptions.TableTypeNotOneErrorException;

import uk.org.ogsadai.client.toolkit.exception.AuthorisationException;
import uk.org.ogsadai.client.toolkit.exception.DataFormatException;
import uk.org.ogsadai.client.toolkit.exception.ResourceUnknownException;
import uk.org.ogsadai.client.toolkit.exception.ServerException;
import uk.org.ogsadai.client.toolkit.exception.ServiceCommsException;
import uk.org.ogsadai.client.toolkit.properties.UnknownPropertyException;

@GridService(name = "DAS", namespace = "http://www.china-vo.org/das/engine", targetPackage = "org.china.vo.das.engine", serviceStyle = "SSTYLE_FACTORY", resourceStyle = "RSTYLE_MAGE", operationProvider = "DestroyProvider, SetTerminationTimeProvider, GetRPProvider, GetMRPProvider, SetRPProvider, QueryRPProvider, SubscribeProvider, GetCurrentMessageProvider, PauseSubscriptionProvider, ResumeSubscriptionProvider")
public class DAS {
	@GridAttribute
	private String status;

	@GridAttribute
	private ExecPlan execplan;

	@GridAttribute
	private int format = 0;

	@GridAttribute
	private String targetURL = null;

	@GridAttribute
	private String queryResult = null;

	@GridAttribute
	private String submitTime = null;

	@GridAttribute
	private String startTime = null;

	@GridAttribute
	private String endTime = null;

	private ADQLParser adqlparser = new ADQLParser();

	private LoopThread loopthread = LoopThread.getInstance();

	/**
	 * 
	 * @param adql
	 * @param format
	 * @param targetURL
	 * @return
	 */
	@GridMethod
	public String synchQuery(String adql, int format) {
		boolean Flag = true;
		this.format = format;

		SelectStatement ss;
		try {
			ss = adqlparser.parse(adql);
			execplan = new ExecPlan(ss);
			this.status = "WAITING";
		} catch (ParserSyntaxException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ParserExpressionException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ADQLErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (DataNodeNotFoundException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (CheckDNTableException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (TableNotFoundErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (GetTableTypeException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (TableTypeNotOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ImageTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (SpecTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (JoinedTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (Exception e) {
			this.status = "Error";
			e.printStackTrace();
		}
		while (Flag) {
//			System.out.println("Session (SynQury), Status is==================:" +this.status);
			if (this.status.equals("COMPLETED") || this.status.equals("Error")) {
				Flag = false;
				return this.queryResult;
			}
			try {
				Thread.sleep(5000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}		
		return this.queryResult;
	}

	/**
	 * 
	 * @param adql
	 * @param format
	 * @param targetURL
	 * @return
	 */
	@GridMethod
	public void AsynchQuery(String adql, int format, String targetURL) {
		this.format = format;
		this.targetURL = targetURL;
		SelectStatement ss;
		try {
			ss = adqlparser.parse(adql);
			execplan = new ExecPlan(ss);
			this.status = "WAITING";
		} catch (ParserSyntaxException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ParserExpressionException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ADQLErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (DataNodeNotFoundException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (CheckDNTableException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (TableNotFoundErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (GetTableTypeException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (TableTypeNotOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (ImageTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (SpecTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (JoinedTableMoreThanOneErrorException e) {
			this.status = "Error";
			e.printStackTrace();
		} catch (Exception e) {
			this.status = "Error";
			e.printStackTrace();
		}
	}

	@GridMethod
	public void start() {
		Thread thread = new Thread(loopthread);
		thread.start();
	}

	@GridMethod
	public String getAllResource() {
		DataResourceMap drm = DataResourceMap.getInstance();
		return drm.getAllResource();
	}

	@GridMethod
	public String getMetaTables(String dataNodeName) {
		DataResourceMap drm = DataResourceMap.getInstance();
		return drm.getMetaTables(dataNodeName);
	}

	@GridMethod
	public String getMetaColumns(String dataNodeName, String tableName) {
		DataResourceMap drm = DataResourceMap.getInstance();
		return drm.getMetaColomns(dataNodeName, tableName);
	}

	/**
	 * @return Returns the execplan.
	 */
	public ExecPlan getExecplan() {
		return execplan;
	}

	/**
	 * @param execplan
	 *            The execplan to set.
	 */
	public void setExecplan(ExecPlan execplan) {
		this.execplan = execplan;
	}

	/**
	 * @return Returns the format.
	 */
	public int getFormat() {
		return format;
	}

	/**
	 * @param format
	 *            The format to set.
	 */
	public void setFormat(int format) {
		this.format = format;
	}

	/**
	 * @return Returns the status.
	 */
	public String getStatus() {
		return status;
	}

	/**
	 * @param status
	 *            The status to set.
	 */
	public void setStatus(String status) {
		this.status = status;
	}

	/**
	 * @return Returns the targetURL.
	 */
	public String getTargetURL() {
		return targetURL;
	}

	/**
	 * @param targetURL
	 *            The targetURL to set.
	 */
	public void setTargetURL(String targetURL) {
		this.targetURL = targetURL;
	}

	public String getEndTime() {
		return endTime;
	}

	public void setEndTime(String endTime) {
		this.endTime = endTime;
	}

	public String getQueryResult() {
		return queryResult;
	}

	public void setQueryResult(String queryResult) {
		this.queryResult = queryResult;
	}

	public String getStartTime() {
		return startTime;
	}

	public void setStartTime(String startTime) {
		this.startTime = startTime;
	}

	public String getSubmitTime() {
		return submitTime;
	}

	public void setSubmitTime(String submitTime) {
		this.submitTime = submitTime;
	}

}