package net.ivoa.adql;

import org.china.vo.das.exceptions.*;
import org.china.vo.das.log.DASLog;
import java.util.*;

public class ADQLOperator extends ADQLToken {
	static final public int op_slash=0x00030000;
	static final public int op_star=0x00030001;
	static final public int op_plus=0x00020002;
	static final public int op_minus=0x00020003;
	static final public int op_gt=0x00010004;
	static final public int op_ge=0x00010005;
	static final public int op_lt=0x00010006;
	static final public int op_le=0x00010007;
	static final public int op_assign=0x00000008;
	static final public int op_noteq=0x00010009;
	
	private int optype=op_slash;
	private ADQLToken od_left;
	private ADQLToken od_right;
	private ADQLOperator parent;
	private int priority;
	public ADQLOperator(int op){
		type=tp_operator;
		optype=op_slash;
		priority=optype&0xffff0000;
		parent=null;
		od_left=null;
		od_right=null;
	}
	public int getPriority(){
		return optype&0xffff0000;
	}
	public ADQLOperator(String op)throws ParserSyntaxException{
		optype=ADQLOperator.convertOpType(op);
		if (optype<0){
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			throw new ParserSyntaxException();
		}
		type=tp_operator;
	}
	static public int convertOpType(String op){
		int optp=-1;
		if(op.equals("=")){
			optp=op_assign;
		}
		else if(op.equals("+")){
			optp=op_plus;
		}
		else if(op.equals("-")){
			optp=op_minus;
		}
		else if(op.equals("*")){
			optp=op_star;
		}
		else if(op.equals("/")){
			optp=op_slash;
		}
		else if(op.equals(">")){
			optp=op_gt;
		}
		else if(op.equals("<")){
			optp=op_lt;
		}
		else if(op.equals(">=")){
			optp=op_ge;
		}
		else if(op.equals("<=")){
			optp=op_le;
		}
		else if(op.equals("!=")){
			optp=op_noteq;
		}
		
		return optp;
	}
	
	public int getOpType(){
		return optype;
	}
	public Object getValue(){
		String ex="";
		if(od_left!=null){
			if(od_left.getClassType()==ADQLToken.tp_operator){
				ex="("+od_left.getValue().toString()+")";
			}
			else if(od_left.getClassType()==ADQLToken.tp_dbobject){
				ex=((DBObject)od_left).getValue().toString();
			}
			else{
				ex=od_left.getValue().toString();
			}
			
		}
		String strop="";
		switch(optype){
		case  op_slash:
			strop="/";
			break;
		case op_star:
			strop="*";
			break;
		case op_plus:
			strop="+";
			break;
		case op_minus:
			strop="-";
			break;
		case op_gt:
			strop=">";
			break;
		case op_ge:
			strop=">=";
			break;
		case op_lt:
			strop="<";
			break;
		case op_le:
			strop="<=";
			break;
		case op_assign:
			strop="=";
			break;
		case op_noteq:
			strop="!=";
			break;
		}
		ex+=strop;
		if(od_right!=null){
			if(od_right.getClassType()==ADQLToken.tp_operator){
				ex+="("+od_right.getValue().toString()+")";
			}
			else if(od_right.getClassType()==ADQLToken.tp_dbobject){
				ex+=((DBObject)od_right).getValue().toString();
			}
			else{
				ex+=od_right.getValue().toString();
			}
		}
		return ex;
	}
	public void setParent(ADQLOperator p){
		parent=p;
	}
	public ADQLOperator getParent(){
		return parent;
	}
	public void setLeft(ADQLToken od){
		if(od.getClassType()==ADQLToken.tp_operator){
			((ADQLOperator)od).setParent(this);
		}
		od_left=od;
	}
	public void setRight(ADQLToken od){
		if(od.getClassType()==ADQLToken.tp_operator){
			((ADQLOperator)od).setParent(this);
		}
		od_right=od;
	}
	public ADQLToken getLeft(){
		return od_left;
	}
	public ADQLToken getRight(){
		return od_right;
	}
	public String getADQLString(){
		String ret="";
		ret=this.getValue().toString();
		return ret;
	}
	public Vector getDBObjectList(){
		Vector lst=new Vector();
		if(od_left!=null){
			if(od_left.getClassType()==ADQLToken.tp_operator){
				Vector lstLeft=((ADQLOperator)od_left).getDBObjectList();
				lst.addAll(lstLeft);
			}
			else if(od_left.getClassType()==ADQLToken.tp_dbobject){
				DBObject lft=(DBObject)od_left;
				if(lft.getSecond(0)!=null){
					lst.add(lft);
				}
				else{
					;
				}
			}
			else if(od_left.getClassType()==ADQLToken.tp_explicitfunction){
				Vector lstLeft=((ExplicitFunction)od_left).getDBObjectList();
				lst.addAll(lstLeft);
			}
		}
		if(od_right!=null){
			if(od_right.getClassType()==ADQLToken.tp_operator){
				Vector lstRight=((ADQLOperator)od_right).getDBObjectList();
				lst.addAll(lstRight);
			}
			else if(od_right.getClassType()==ADQLToken.tp_dbobject){
				DBObject rt=(DBObject)od_right;
				if(rt.getSecond(0)!=null){
					lst.add(rt);
				}
				else{
					;
				}
			}
			else if(od_right.getClassType()==ADQLToken.tp_explicitfunction){
				Vector lstLeft=((ExplicitFunction)od_right).getDBObjectList();
				lst.addAll(lstLeft);
			}
		}
		return lst;
	}
	public Vector getAliasList(){
		Vector dblist=getDBObjectList();
		Vector list=new Vector();
		for(int i=0;i<dblist.size();i++){
			list.add(((DBObject)dblist.get(i)).getFirst().getADQLString());
		}
		return list;
	}
	public void replaceAliasName(String oname, String nname){
		if(od_left!=null){
			if(od_left.getClassType()==ADQLToken.tp_operator){
				((ADQLOperator)od_left).replaceAliasName(oname,nname);
			}
			else if(od_left.getClassType()==ADQLToken.tp_dbobject){
				DBObject lft=(DBObject)od_left;
				if(lft.getSecond(0)!=null && lft.getFirst().getADQLString().equals(oname)){
					lft.rename(nname);
				}
				else{
					;
				}
			}
			else if(od_left.getClassType()==ADQLToken.tp_explicitfunction){
				((ExplicitFunction)od_left).replaceAliasName(oname,nname);
			}
		}
		if(od_right!=null){
			if(od_right.getClassType()==ADQLToken.tp_operator){
				((ADQLOperator)od_right).replaceAliasName(oname,nname);
			}
			else if(od_right.getClassType()==ADQLToken.tp_dbobject){
				DBObject rt=(DBObject)od_right;
				if(rt.getSecond(0)!=null && rt.getFirst().getADQLString().equals(oname)){
					((DBObject)rt).rename(nname);
				}
				else{
					;
				}
			}
			else if(od_right.getClassType()==ADQLToken.tp_explicitfunction){
				((ExplicitFunction)od_right).replaceAliasName(oname,nname);
			}
		}
	}
}
