package net.ivoa.adql;

import java.util.Vector;

import net.ivoa.adql.exceptions.DBObjectNotFoundException;
import net.ivoa.adql.exceptions.TableItemNotFoundException;

public class Column extends SelectItem {
	ADQLToken dbo;
	DBObject alias=null;
	public Column(ADQLToken d){
		dbo=d;
	}
	public void setAlias(DBObject d){
		alias=d;
	}
	/**
	 * @return 1:only contains alias in tableAliasList
	 * 				2:contains other alias not in tableAliasList
	 * 				3:exception	
	 * 				0:not contains alias in tableAliasList
	 */
	public int containsTables(Vector tableAliasList)throws TableItemNotFoundException
	{
		int ret=0;
		switch(dbo.getClassType()){
		case ADQLToken.tp_dbobject:
			String dboas=((DBObject)dbo).getFirst().getADQLString();
			for(int i=0;i<tableAliasList.size();i++){
				DBObject a=(DBObject)tableAliasList.get(i);
				String af=a.getFirst().getADQLString();
				if(af.equals(dboas)){
					ret=1;
					break;
				}
			}
			break;
		case ADQLToken.tp_operator:
			Vector dboaslist=((ADQLOperator)dbo).getAliasList();
			if(dboaslist!=null){
				int flag=0;
				for(int i=0;i<dboaslist.size();i++){
					String o=(String)dboaslist.get(i);
					for(int j=0;j<tableAliasList.size();j++){
						String a=((DBObject)tableAliasList.get(j)).getFirst().getADQLString();
						if(a.equals(o)){
							flag++;
						}
					}
				}
				if(flag<dboaslist.size() && flag>0){
					ret=2;
				}
				else if(flag==dboaslist.size() && flag>0){
					ret=1;
				}
				else if(flag==0){
					ret=0;
				}
				else{
					ret=3;
				}
			}
			break;
		}
		
		return ret;
	}
	/**
	 * listup all DBObjects contains in the columns and return as a Vector object
	 * @return
	 * @throws DBObjectNotFoundException
	 */
	private Vector getAllDBObjects0()throws DBObjectNotFoundException{
		Vector ret=null;
		switch(dbo.getClassType()){
		case ADQLToken.tp_dbobject:
			ret=new Vector();
			ret.add(dbo);
			break;
		case ADQLToken.tp_operator:
			ret=((ADQLOperator)dbo).getDBObjectList();			
			break;
		}
		return ret;
	}
	/**
	 * list up all DBObjects contain in the Column and return as an array
	 */
	public DBObject[] getAllDBObjects()throws DBObjectNotFoundException{
		DBObject[] ret=null;
		Vector ret0=getAllDBObjects0();
		if(ret0!=null && ret0.size()>0){
			ret=new DBObject[ret0.size()];
			for(int i=0;i<ret0.size();i++){
				ret[i]=(DBObject)ret0.get(i);
			}
		}
		return ret;
	}
	/**
	 * Query all DBObjects which are not contained in vector currDNTableAlias
	 * @return an array of DBObjects
	 */
	public DBObject[] getAllDBObjectsWithoutCurrDN(Vector currDNTableAlias)throws TableItemNotFoundException, DBObjectNotFoundException
	{
		DBObject[] ret=null;
		Vector ret0=getAllDBObjects0();
		if(ret0!=null){
			for(int i=0;i<ret0.size();i++){
				String o=((DBObject)ret0.get(i)).getFirst().getADQLString();
				for(int j=0;j<currDNTableAlias.size();j++){
					String a=((DBObject)currDNTableAlias.get(j)).getFirst().getADQLString();
					if(a.equals(o)){
						ret0.remove(i);
						break;
					}
				}
			}
			ret=new DBObject[ret0.size()];
			for(int i=0;i<ret.length;i++){
				ret[i]=(DBObject)ret0.get(i);
			}
		}
		return ret;
	}
	/**
	 * get the column's name string, can be used in SQL query string
	 * @return 
	 */
	public String getADQLString(){
		String ret="";
		ret=dbo.getADQLString();
		if(alias!=null){
			ret=ret+" as "+alias.getADQLString();
		}
		return ret;
	}
	/**
	 * using newname to take place the oldname forall DBObjects containsin the column
	 * @param oldname
	 * @param newname
	 */
	public void replaceName(String oldname, String newname){
		if(dbo!=null && dbo.getClassType()==ADQLToken.tp_dbobject){			
			if(((DBObject)dbo).getSecond(0)!=null && ((DBObject)dbo).getFirst().getADQLString().equals(oldname) ){
				((DBObject)dbo).rename(newname);
			}
			else if(dbo!=null && dbo.getClassType()==ADQLToken.tp_operator){
				((ADQLOperator)dbo).replaceAliasName(oldname,newname);
			}
		}
	}
	/**
	 * using newname to take place the oldname forall DBObjects containsin the column
	 * @param oldname
	 * @param newname
	 */
	public void replaceAlias(String oldname, String newname){
		if(alias!=null){
			dbo=alias;alias=null;				
		}
		else{
			replaceName(oldname,newname);
			}
	}
	public DBObject getAlias(){
		return alias;
	}
}
