package org.china.vo.das.converters.resultset;

import java.sql.Types;

import uk.org.ogsadai.converters.resultset.ColumnStrategy;
import uk.org.ogsadai.converters.resultset.ColumnStrategyFactory;
import uk.org.ogsadai.converters.resultset.webrowset.*;

/**
 * A factory that creates column strategies for converting result sets into the
 * WebRowSet XML format.
 * 
 * @author The OGSA-DAI Project Team
 */
public class VOTableColumnStrategyFactory implements ColumnStrategyFactory
{
    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) The University of Edinburgh 2002 - 2005.";

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ColumnStrategyFactory.createColumnStrategy(int)
     */
    public ColumnStrategy createColumnStrategy(final int sqlType)
    {
        ColumnStrategy strategy = null;
        switch (sqlType)
        {
            // Add appropriate strategies for each type.
            case Types.BIGINT:
            case Types.BIT:
              case Types.BOOLEAN:
            case Types.DECIMAL:
            case Types.DOUBLE:
            case Types.FLOAT:
            case Types.INTEGER:
            case Types.NUMERIC:
            case Types.REAL:
            case Types.SMALLINT:
            case Types.TINYINT:
                strategy = new NumberColumnStrategy();
                break;
 
            case Types.CHAR:
            case Types.CLOB:
            case Types.LONGVARCHAR:
            case Types.VARCHAR:
                strategy = new TextColumnStrategy();
                break;
        
            case Types.BINARY:
            case Types.BLOB:
            case Types.LONGVARBINARY:
            case Types.VARBINARY:
                strategy = new BinaryColumnStrategy();
                break;
            
            case Types.DATE:
            case Types.TIME:
            case Types.TIMESTAMP:
                strategy = new TextColumnStrategy();
                break;
                
            case Types.ARRAY:
            case Types.DISTINCT:
            case Types.JAVA_OBJECT:
            case Types.NULL:
            case Types.OTHER:
            case Types.REF:
            case Types.STRUCT:
                strategy = new UnsupportedColumnStrategy();
                break;
            
            default:
                strategy = new UnknownColumnStrategy();
        }
        return strategy;
    }
}