package org.china.vo.das.parser;
import net.ivoa.adql.*;
import net.ivoa.adql.exceptions.*;
import java.util.*;
import org.china.vo.das.log.*;
import org.china.vo.das.exceptions.*;

public class ADQLParser {
	
	/**
	 * parser status
	 */
	final private int st_at1=0;
	final private int st_at2=1;
	final private int st_bif=2;
	final private int st_cl1=3;
	final private int st_cl2=4;
	final private int st_cm1=5;
	final private int st_cm2=6;
	final private int st_db1=7;
	final private int st_db2=8;
	final private int st_dq1=9;
	final private int st_dq2=10;
	final private int st_end=-2;
	final private int st_err=-1;
	final private int st_lit=11;	
	final private int st_nul=12;
	final private int st_num=13;
	final private int st_op1=14;
	final private int st_qlt=15;
	final private int st_rl1=16;
	final private int st_rl2=17;
	final private int st_sla=18;
	final private int st_sta=19;
	final private int st_str=20;
	final private int st_sn1=21;
	final private int st_sn2=22;
	final private int st_sn3=23;
	final private int st_svr=24;
	final private int st_tbc=25;	
	final private int st_var=26;
	
	final private int est_nul=0;
	final private int est_od=1;
	final private int est_op1=2;
	final private int est_op2=3;
	final private int est_sta=4;
	final private int est_setop=5;
	final private int est_end=-1;
	final private int est_err=6;
	/**
	 * current status
	 */
	private int s=st_nul;
	private int exprs=est_nul;
	/**
	 * current token string
	 */
	private String chs="";
	/**
	 * Token object
	 */
	private ADQLToken tkn=null;
	/**
	 * ADQL string current position index from the beginning of ADQL string.
	 * The first char of ADQL is 0.
	 */
	private int m_nOffset=0;
	/**
	 * ADQL string previous position offset. it begins at m_nOffset backword. 
	 */
	private int m_nPref=1;
	/**
	 * A flag to encounter params
	 */
	private int m_nLParam=0;
	/**
	 * delimiter tokens selection. items are ADQLToken object
	 */
	private LinkedList m_strDelimiter=null;
	/**
	 * ADQL string
	 */
	private String m_strAdql=null;
	/**
	 *turn back to previous position of ADQL string
	 *
	 */
	
	private HashMap keywords;
	private HashMap functions;
	
	public ADQLParser(){
		keywords=new HashMap(200);
		functions=new HashMap();
		keywords.put("add","add");
		keywords.put("all","all");
		keywords.put("alter","alter");
		keywords.put("and","and");
		keywords.put("any","any");
		keywords.put("as","as");
		keywords.put("asc","asc");
		keywords.put("add","add");
		keywords.put("authorization","authorization");
		keywords.put("auto","auto");
		keywords.put("backup","backup");
		keywords.put("base64","base64");
		keywords.put("begin","begin");
		keywords.put("between","between");
		keywords.put("binary","binary");
		keywords.put("break","break");
		keywords.put("browse","browse");
		keywords.put("bulk","bulk");
		keywords.put("by","by");
		keywords.put("cascade","cascade");
		keywords.put("case","case");
		keywords.put("cast","cast");
		keywords.put("check","check");
		keywords.put("checkpoint","checkpoint");
		keywords.put("close","close");
		keywords.put("clustered","clustered");
		keywords.put("coalesce","coalesce");
		keywords.put("collate","collate");
		keywords.put("column","column");
		keywords.put("commit","commit");
		keywords.put("compute","compute");
		keywords.put("concat","concat");
		keywords.put("constraint","constraint");
		keywords.put("contains","contains");
		keywords.put("containstable","containstable");
		keywords.put("continue","continue");
		keywords.put("create","create");
		keywords.put("cross","cross");
		keywords.put("cube","cube");
		keywords.put("current","current");
		keywords.put("current_date","current_date");
		keywords.put("current_time","current_time");
		keywords.put("current_timestamp","current_timestamp");
		keywords.put("current_user","current_user");
		keywords.put("cursor","cursor");
		keywords.put("database","database");
		keywords.put("dbcc","dbcc");
		keywords.put("deallocate","deallocate");
		keywords.put("declare","declare");
		keywords.put("default","default");
		keywords.put("delete","delete");
		keywords.put("deny","deny");
		keywords.put("desc","desc");
		keywords.put("disk","disk");
		keywords.put("distinct","distinct");
		keywords.put("distributed","distributed");
		keywords.put("double","double");
		keywords.put("drop","drop");
		keywords.put("dump","dump");
		keywords.put("elements","elements");
		keywords.put("else","else");
		keywords.put("end","end");
		keywords.put("errlvl","errllvl");
		keywords.put("escape","escape");
		keywords.put("except","except");
		keywords.put("exec","exec");
		keywords.put("execute","execute");
		keywords.put("exists","exists");
		keywords.put("exit","exit");
		keywords.put("expand","expand");
		keywords.put("explicit","explicit");
		keywords.put("fast","fast");
		keywords.put("fastfirstrow","fastfirstrow");
		keywords.put("fetch","fetch");
		keywords.put("file","file");
		keywords.put("fillfactor","fillfactor");
		keywords.put("for","for");
		keywords.put("force","force");
		keywords.put("foreign","foreign");
		keywords.put("freetext","freetext");
		keywords.put("freetexttable","freetexttable");
		keywords.put("from","from");
		keywords.put("full","full");
		keywords.put("function","function");
		keywords.put("goto","goto");
		keywords.put("grant","grant");
		keywords.put("group","group");
		keywords.put("hash","hash");
		keywords.put("having","having");
		keywords.put("holdlock","holdlock");
		keywords.put("identity","identity");
		keywords.put("identity_insert","identity_insert");
		keywords.put("identitycol","identitycol");
		keywords.put("if","if");
		keywords.put("in","in");
		keywords.put("index","index");
		keywords.put("inner","inner");
		keywords.put("insert","insert");
		keywords.put("intersect","intersect");
		keywords.put("into","into");
		keywords.put("is","is");
		keywords.put("join","join");
		keywords.put("keep","keep");
		keywords.put("keepfixed","keepfixed");
		keywords.put("key","key");
		keywords.put("kill","kill");
		keywords.put("left","left");
		keywords.put("like","like");
		keywords.put("lineno","lineno");
		keywords.put("load","load");
		keywords.put("loop","loop");
		keywords.put("maxdop","maxdop");
		keywords.put("merge","merge");
		keywords.put("national","national");
		keywords.put("nocheck","nocheck");
		keywords.put("nolock","nolock");
		keywords.put("nonclustered","nonclustered");
		keywords.put("not","not");
		keywords.put("null","null");
		keywords.put("of","of");
		keywords.put("off","off");
		keywords.put("offsets","offsets");
		keywords.put("on","on");
		keywords.put("open","open");
		keywords.put("opendatasource","opendatasource");
		keywords.put("openquery","openquery");
		keywords.put("openrowset","openrowset");
		keywords.put("openxml","opemxml");
		keywords.put("option","option");
		keywords.put("or","or");
		keywords.put("order","order");
		keywords.put("outer","outer");
		keywords.put("over","over");
		keywords.put("paglock","paglock");
		keywords.put("percent","percent");
		keywords.put("plan","plan");
		keywords.put("precision","precision");
		keywords.put("primary","primary");
		keywords.put("print","print");
		keywords.put("proc","proc");
		keywords.put("procedure","procedure");
		keywords.put("public","public");
		keywords.put("raiserror","raiserror");
		keywords.put("raw","raw");
		keywords.put("read","read");
		keywords.put("readcommited","readcommited");
		keywords.put("readpast","readpast");
		keywords.put("readtext","readtext");
		keywords.put("readuncommited","readuncommited");
		keywords.put("reconfigure","reconfigure");
		keywords.put("references","references");
		keywords.put("region","region");
		keywords.put("remote","remote");
		keywords.put("repeatableread","repeatableread");
		keywords.put("replication","replicatiion");
		keywords.put("restore","restore");
		keywords.put("restrict","restrict");
		keywords.put("return","return");
		keywords.put("revoke","revoke");
		keywords.put("right","right");
		keywords.put("robust","robust");
		keywords.put("rollback","rollback");
		keywords.put("rollup","rollup");
		keywords.put("rowcount","rowcount");
		keywords.put("rowguidcol","rowguidcol");
		keywords.put("rowlock","rowlock");
		keywords.put("rule","rule");
		keywords.put("save","save");
		keywords.put("schema","schema");
		keywords.put("select","select");
		keywords.put("serializable","serializable");
		keywords.put("session_user","session_user");
		keywords.put("set","set");
		keywords.put("setuser","setuser");
		keywords.put("shutdown","shutdown");
		keywords.put("some","some");
		keywords.put("statistics","statistics");
		keywords.put("system_user","system_user");
		keywords.put("table","table");
		keywords.put("tablock","tablock");
		keywords.put("tablockx","tablockx");
		keywords.put("textsize","textsize");
		keywords.put("then","then");
		keywords.put("ties","ties");
		keywords.put("to","to");
		keywords.put("top","top");
		keywords.put("tran","tran");
		keywords.put("transaction","transaction");
		keywords.put("trigger","trigger");
		keywords.put("tsequal","tsequal");
		keywords.put("union","union");
		keywords.put("unique","unique");
		keywords.put("update","update");
		keywords.put("updatetext","updatetext");
		keywords.put("updlock","updlock");
		keywords.put("use","use");
		keywords.put("user","user");
		keywords.put("values","values");
		keywords.put("varying","varying");
		keywords.put("view","view");
		keywords.put("views","views");
		keywords.put("waitfor","waitfor");
		keywords.put("when","when");
		keywords.put("where","where");
		keywords.put("while","while");
		keywords.put("with","with");
		keywords.put("writetext","writetext");
		keywords.put("xlock","xlock");
		keywords.put("xmatch","xmatch");
		keywords.put("xml","xml");
		keywords.put("xmldata","xmldata");
		
		functions.put("region","region");
		functions.put("xmatch","xmatch");
		functions.put("acos","acos");
		functions.put("asin","asin");
		functions.put("atan","atan");
		functions.put("atan2","atan2");
		functions.put("cos","cos");
		functions.put("cot","cot");
		functions.put("sin","sin");
		functions.put("tan","tan");
		functions.put("abs","abs");
		functions.put("ceiling","ceiling");
		functions.put("degrees","degrees");
		functions.put("exp","exp");
		functions.put("floor","floor");
		functions.put("log","log");
		functions.put("log10","log10");
		functions.put("mod","mod");
		functions.put("pi","pi");
		functions.put("power","power");
		functions.put("radians","radians");
		functions.put("sqrt","sqrt");
		functions.put("rand","rand");
		functions.put("round","round");
		functions.put("truncate","truncate");
		//for test
		functions.put("func","func");
	}
	private void rewind(){
		if(m_nOffset-m_nPref>=0){
			m_nOffset-=m_nPref;
			m_nPref=1;
		}
		else{
			m_nOffset=0;
			m_nPref=1;
		}
	}
	/**
	 *  get current char from ADQL string
	 * @return current charactor
	 */
	private char getChar(){
		return m_strAdql.charAt(m_nOffset++);
	}
	/**
	 * if the character is a letter it returns true, otherwise false.
	 * @param ch current character
	 * @return true: ch is a letter false: ch is not a letter.
	 */
	private boolean isLetter(char ch){
		if((ch>='a' && ch<='z') || (ch>='A' && ch<='Z') || ch=='_' || ch=='#'){
			return true;
		}
		else{
			return false;
		}
	}
	/**
	 * if the character is a digit it returns true, otherwise false.
	 * @param ch current character
	 * @return true: ch is a digit false: ch is not a digit.
	 */
	private boolean isDigit(char ch){
		if(ch>='0' && ch<='9'){
			return true;
		}
		else{
			return false;
		}
	}
	/**
	 * if the character ch is an operator return true, otherwise false
	 * @param ch: character
	 * @param withstar: true-star * is treated  as an operator, false-exclude star *
	 * @param withslash: true-slash / is treated as an operator, false-exclude slash /
	 * @return true: ch is an operator, false:ch is not an operator
	 */
	private boolean isOperator(char ch, boolean withstar, boolean withslash){
		if(ch=='=' || ch=='>' || ch=='<' || ch=='!' || ch=='+' || ch=='-' || ch=='%' || ch=='~' ||ch=='|' || ch=='^' || ch=='&'){
			return true;
		}
		else if (withstar && ch=='*'){
			return true;
		}
		else if (withslash && ch=='/'){
			return true;
		}
		else{
			return false;
		}
	}
	private void trim(){
		char c=getChar();
		while(c==' '){
			c=getChar();
		}
		if(c!=' '){
			rewind();
		}
	}
	/**
	 * parse when in status nul
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_nul(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)){
			chs+=ch;
			s=st_lit;
		}
		else if(isDigit(ch)){
			chs+=ch;
			s=st_num;
		}
		else if(ch==' '){
			trim();
			s=st_nul;
		}
		else if(ch=='@'){
			chs+=ch;
			s=st_at1;
		}
		else if(ch=='\''){
			s=st_str;
		}
		else if(ch=='/'){
			chs+=ch;
			s=st_sla;
		}
		else if(ch==':'){
			chs+=ch;
			s=st_cl1;
		}
		else if(ch=='*'){
			chs+=ch;
			s=st_sta;
		}
		else if(isOperator(ch,false,false)){
			chs+=ch;
			s=st_op1;
		}
		else if(ch==';'){
			tkn=new Eof();
			s=st_end;
		}
		else if(ch=='('){
			tkn=new LParam();
			s=st_end;
		}
		else if(ch==')'){
			tkn=new RParam();
			s=st_end;
		}
		else if(ch==','){
			tkn=new Comma();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * if it is EOF returns true
	 * @return true:EOF otherwise false.
	 */
	private boolean isEOF(){
		if(m_strAdql.length()<=m_nOffset){
			return true;
		}
		else{
			return false;
		}
	}
	/**
	 * detect if a string is a keyword
	 * @param chs
	 * @return
	 */
	private boolean isKeyword(String chs){
		String chslc=chs.toLowerCase();
		return keywords.containsKey(chslc);
	}
	/**
	 * detect if a string is a function name
	 * @param chs
	 * @return
	 */
	private boolean isFunction(String chs){
		String chslc=chs.toLowerCase();
		return functions.containsKey(chslc);
	}
	/**
	 * parse adql when in status lit
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_lit(char ch)throws ParserSyntaxException,ParserExpressionException{
		ADQLToken tkn=null;
		if(isLetter(ch) || isDigit(ch)){
			chs+=ch;
			s=st_lit;
		}
		else if(ch=='.'){
			//chs+=ch;
			tkn=new DBObject();
			Identifier id=new Identifier();
			id.setValue(chs);
			chs="";
			((DBObject)tkn).addFirst(id);
			s=st_db1;
		}
		else if(ch==' '){
			trim();
			if(isKeyword(chs)){
				tkn=new Keyword(chs);				
			}
			else{
				tkn=new DBObject();
				Identifier id=new Identifier();
				id.setValue(chs);
				chs="";
				((DBObject)tkn).addFirst(id);
			}
			s=st_end;
		}		
		else if(ch==':'){
			ArchiveTable at=new ArchiveTable();
			at.setValue(chs);
			tkn=new ArchiveTableItem();
			((ArchiveTableItem)tkn).setArchiveTable(at);
			chs="";
			ADQLToken dbo=parseToken();
			if(dbo.getClassType()==ADQLToken.tp_dbobject){
				((ArchiveTableItem)tkn).setTableName((DBObject)dbo);
				s=st_end;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}			
		}
		else if(isOperator(ch,true,true)||ch==','||ch==';'){
			tkn=new DBObject();
			Identifier id=new Identifier();
			id.setValue(chs);
			chs="";
			((DBObject)tkn).addFirst(id);
			rewind();
			s=st_end;
		}
		else if(ch==')'){
			tkn=new DBObject();
			Identifier id=new Identifier();
			id.setValue(chs);
			chs="";
			((DBObject)tkn).addFirst(id);		
			rewind();
			s=st_end;
		}/*
		else if(ch==';'){
			tkn=new Eof();
			s=st_end;
		}*/
		else if(ch=='('){
			if(isFunction(chs)){
				ExplicitFunction exf=new ExplicitFunction();
				exf.setName(chs);
				Expression param=null;
				int  term=ADQLToken.tp_eof;
				int flag=1;
				ADQLParser pa2=new ADQLParser();
				pa2.m_strAdql=this.m_strAdql.substring(this.m_nOffset);
				do{				
					pa2.exprs=est_nul;
					param=pa2.parseExpression();					
					if(param!=null){
						exf.addParameter(param);
						ADQLToken t=param.terminator;						
						term=t.getClassType();
						if(term==ADQLToken.tp_comma){
							flag=1;
						}
						else if(term==ADQLToken.tp_rparam){
							flag=0;
						}
						else{
							flag=-1;
						}
					}
					else{
						flag=-1;
					}
				}while(flag>0);
				//this.m_nOffset+=pa2.m_nOffset-1;
				this.m_nOffset+=pa2.m_nOffset;
				if(flag<0){
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					s=st_err;
				}
				else{
					tkn=exf;
					s=st_end;
				}
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}
		}	
		else{//ch=='@' || ch=='\'' || ch=='*'  etc.			
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is str
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_str(char ch){
		ADQLToken tkn=null;
		if(ch=='\''){
			tkn=new DBObject();
			QuoteIdentifier id=new QuoteIdentifier();
			id.setValue(chs);
			((DBObject)tkn).addFirst(id);
			s=st_qlt;
		}
		else{
			chs+=ch;
			s=st_str;
		}
		return tkn;
	}
	/**
	 * parse number
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_num(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_num;
		}
		else if(ch=='e' || ch=='E'){
			chs+=ch;
			s=st_sn1;
		}
		else if(ch=='.'){
			chs+=ch;
			s=st_rl1;
		}
		else if(ch==' ' || ch==',' ||ch==')'||ch==';'||isOperator(ch,true,true)){
			if(ch==' ' ){
				trim();
			}
			tkn=new ADQLInteger();
			tkn.setValue(chs);
			s=st_end;
			if(isOperator(ch,true,true)|| ch==',' ||ch==')'||ch==';'){//updated by LC 20061114
				rewind();
			}
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse wheen in status at1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_at1(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)){
			chs+=ch;
			s=st_var;
		}
		else if(ch=='@'){
			chs+=ch;
			s=st_at2;
		}
		else if(ch=='\''){
			s=st_str;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when in status at2
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_at2(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)){
			chs+=ch;
			s=st_svr;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when in status sla
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_sla(char ch){
		ADQLToken tkn=null;
		/*if(isLetter(ch)){
			chs+=ch;
			s=st_lit;
		}//Modified by LC 20071113
		else*/ if(isDigit(ch) || ch=='('||isLetter(ch)||ch=='\''){
			tkn=new ADQLOperator(ADQLOperator.op_slash);
			chs="";
			rewind();
			s=st_end;
		}
		else if(ch=='*'){
			s=st_cm1;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when in status cl1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_cl1(char ch){
		ADQLToken tkn=null;
		if(ch==':'){			
			s=st_cl2;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	private ADQLToken parseInStatus_cl2(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)){
			chs+=ch;
			s=st_bif;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when in status op1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_op1(char ch)throws ParserSyntaxException{
		ADQLToken tkn=null;
		if(isDigit(ch) || ch=='('){
			tkn=new ADQLOperator(chs);
			rewind();
			s=st_end;
		}
		else if(isOperator(ch,true,true)){
			if(chs.equals(">")){
				if(ch=='='){
					chs+=ch;
					tkn=new ADQLOperator(chs);
					s=st_end;
				}
				else{
					tkn=new ADQLOperator(chs);
					rewind();
					s=st_end;
				}
			}
			else if(chs.equals("<")){
				if(ch=='=' || ch=='>'){
					chs+=ch;
					tkn=new ADQLOperator(chs);
					s=st_end;
				}
				else{
					tkn=new ADQLOperator(chs);
					rewind();
					s=st_end;
				}
			}
			else if(chs.equals("!")){
				if(ch=='>'||ch=='<'||ch=='='){
					chs+=ch;
					tkn=new ADQLOperator(chs);
					s=st_end;
				}
				else{
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					s=st_err;
				}
			}
			else{
				
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}
		}
		else{
			tkn=new ADQLOperator(chs);
			rewind();
			s=st_end;
		}
		return tkn;
	}
	/**
	 * parse when status is sta
	 * @param ch
	 * @return
	 * @throws ParserSyntaxException
	 */
	private ADQLToken parseInStatus_sta(char ch)throws ParserSyntaxException{
		ADQLToken tkn=null;
		if(isLetter(ch)||isDigit(ch)||ch=='@'||ch=='\''||ch=='('){
			tkn=new ADQLOperator(chs);
			rewind();
			s=st_end;
		}
		else if(ch==' '){
			trim();
			tkn=new Star();
			s=st_end;
		}
		else if(ch==','||ch==';'||ch==')'){
			tkn=new Star();
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is db1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_db1(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)){
			chs+=ch;
			s=st_db2;
		}
		else if(ch=='\''){
			s=st_dq1;
		}
		else if(ch=='*'){
			s=st_tbc;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is db2
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_db2(DBObject dbo, char ch){
		ADQLToken tkn=null;
		if(isLetter(ch) || isDigit(ch)){
			chs+=ch;
			s=st_db2;
		}
		else if(ch==' '){
			trim();
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				s=st_end;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}			
		}
		else if(ch==',' || ch==')' || isOperator(ch,true,true)|| ch==';'){
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				rewind();
				s=st_end;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}	
		}
		else if(ch=='.'){
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				s=st_db1;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}	
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is dq1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_dq1(char ch){
		ADQLToken tkn=null;
		if(ch=='\''){
			s=st_dq2;
		}
		else{
			chs+=ch;
			s=st_dq1;
		}
		return tkn;
	}
	/**
	 * parse when status is dq2
	 * @param dbo
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_dq2(DBObject dbo, char ch){
		ADQLToken tkn=null;
		if(ch==' '){
			trim();
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				s=st_end;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}		
		}
		else if(isOperator(ch,true,true) || ch==';'||ch==')'||ch==','){
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				rewind();
				s=st_end;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}
		}
		else if(ch=='.'){
			if(dbo!=null){
				Identifier id=new Identifier();
				id.setValue(chs);
				dbo.addSecond(id);
				s=st_db1;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}			
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is tbc
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_tbc(char ch){
		ADQLToken tkn=null;
		if(ch==' '){			
			trim();
			tkn=new TableColumns();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(isOperator(ch,true,true)||ch==','||ch==';'||ch==')'){
			tkn=new TableColumns();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is qlt
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_qlt(char ch){
		ADQLToken tkn=null;
		if(ch==' ' ||ch=='.' || isOperator(ch,true,true)||ch==')'||ch==';'||ch==','){
			
			QuoteIdentifier id=new QuoteIdentifier();
			id.setValue(chs);
			tkn=new DBObject();
			((DBObject)tkn).addFirst(id);
			if(ch=='.'){
				s=st_db1;
			}
			else{
				rewind();
				s=st_end;
			}
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is sn1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_sn1(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_sn2;
		}
		else if(ch=='+' || ch=='-'){
			chs+=ch;
			s=st_sn3;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is sn2
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_sn2(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_sn2;
		}
		else if(ch==' '){
			trim();
			tkn=new ADQLReal();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(ch==',' || ch==')' || ch==';' || isOperator(ch,true,true)){
			tkn=new ADQLReal();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is sn3
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_sn3(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_sn2;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is cm1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_cm1(char ch){
		ADQLToken tkn=null;
		if(ch=='*'){
			chs+=ch;
			s=st_cm2;
		}
		else{
			chs+=ch;
			s=st_cm1;
		}
		return tkn;
	}
	/**
	 * parse when status is cm2
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_cm2(char ch){
		ADQLToken tkn=null;
		if(ch=='/'){
			tkn=new ADQLComment();
			tkn.setValue(chs.substring(0,chs.length()-2));
			s=st_end;
		}
		else if(ch=='*'){
			chs+=ch;
			s=st_cm2;
		}
		else{
			chs+=ch;
			s=st_cm1;
		}
		return tkn;
	}
	/**
	 * parse when status is rl1
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_rl1(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_rl2;
		}
		else if(ch=='e'||ch=='E'){
			chs+=ch;
			s=st_sn1;
		}
		else if(ch==' '){
			trim();
			tkn=new ADQLReal();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(isOperator(ch,true,true)||ch==';'||ch==')'||ch==','){
			tkn=new ADQLReal();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is rl2
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_rl2(char ch){
		ADQLToken tkn=null;
		if(isDigit(ch)){
			chs+=ch;
			s=st_rl2;
		}
		else if(ch=='e'||ch=='E'){
			chs+=ch;
			s=st_sn1;
		}
		else if(ch==' '){
			trim();
			tkn=new ADQLReal();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(isOperator(ch,true,true)||ch==';'||ch==')'||ch==','){
			tkn=new ADQLReal();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse when status is var
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_var(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)||isDigit(ch)){
			chs+=ch;
			s=st_var;
		}
		else if(ch==' '){
			trim();
			tkn=new ADQLVariable();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(ch==','||isOperator(ch,true,true)||ch==';'||ch==')'){
			tkn=new ADQLVariable();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * parse wen status is svr
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_svr(char ch){
		ADQLToken tkn=null;
		if(isLetter(ch)||isDigit(ch)){
			chs+=ch;
			s=st_svr;
		}
		else if(ch==' '){
			trim();
			tkn=new ADQLSysVariable();
			tkn.setValue(chs);
			s=st_end;
		}
		else if(ch=='@'){
			chs+=ch;
			s=st_at1;
		}
		else if(isOperator(ch,true,true)||ch==';'||ch==')'||ch==','){
			tkn=new ADQLSysVariable();
			tkn.setValue(chs);
			rewind();
			s=st_end;
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	/**
	 * not completed
	 * @param chs
	 * @return
	 */
	private boolean isBIFunction(String chs){
		return false;
	}
	/**
	 * parse when status is bif
	 * @param ch
	 * @return
	 */
	private ADQLToken parseInStatus_bif(char ch)throws ParserSyntaxException,ParserExpressionException{
		ADQLToken tkn=null;
		if(isLetter(ch)||isDigit(ch)){
			chs+=ch;
			s=st_bif;
		}
		else if(ch=='('){
			if(isBIFunction(chs)){
				ADQLBuiltInFunction bif=new ADQLBuiltInFunction();
				Expression param=null;
				int  term=ADQLToken.tp_eof;
				int flag=1;
				do{
					param=parseExpression();
					if(param!=null){
						bif.addParameter(param);
						ADQLToken t=parseToken();
						term=t.getClassType();
						if(term==ADQLToken.tp_comma){
							flag=1;
						}
						else if(term==ADQLToken.tp_rparam){
							flag=0;
						}
						else{
							flag=-1;
						}
					}
					else{
						flag=-1;
					}
				}while(flag>0);
				if(flag<0){
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					s=st_err;				
				}
				else{
					tkn=bif;
					s=st_end;
				}		
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				s=st_err;
			}
		}
		else{
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			s=st_err;
		}
		return tkn;
	}
	
	/**
	 * scan ADQL and get the next token object
	 * @return ADQLToken object
	 */
	private ADQLToken parseToken()throws ParserSyntaxException,ParserExpressionException{			
		ADQLToken tkn=null;
		ADQLToken tkn0=null;
		char ch=getChar();
		do{			
			if(s==st_nul){			
				tkn0=parseInStatus_nul(ch);				
			}
			else if(s==st_lit){
				tkn0=parseInStatus_lit(ch);
			}
			else if(s==st_num){
				tkn0=parseInStatus_num(ch);
			}
			else if(s==st_err){
				throw new ParserSyntaxException();
			}
			else if(s==st_str){
				tkn0=parseInStatus_str(ch);
			}
			else if(s==st_at1){
				tkn0=parseInStatus_at1(ch);
			}
			else if(s==st_at2){
				tkn0=parseInStatus_at2(ch);
			}
			else if(s==st_sla){
				tkn0=parseInStatus_sla(ch);
			}
			else if(s==st_cl1){
				tkn0=parseInStatus_cl1(ch);
			}
			else if(s==st_cl2){
				tkn0=parseInStatus_cl2(ch);
			}
			else if(s==st_op1){
				tkn0=parseInStatus_op1(ch);
			}
			else if(s==st_sta){
				tkn0=parseInStatus_sta(ch);
			}
			else if(s==st_db1){
				tkn0=parseInStatus_db1(ch);
			}
			else if(s==st_db2){
				tkn0=parseInStatus_db2((DBObject)tkn,ch);
			}
			else if(s==st_dq1){
				tkn0=parseInStatus_dq1(ch);
			}
			else if(s==st_dq2){
				tkn0=parseInStatus_dq2((DBObject)tkn,ch);
			}
			else if(s==st_tbc){
				tkn0=parseInStatus_tbc(ch);
			}
			else if(s==st_qlt){
				tkn0=parseInStatus_qlt(ch);
			}
			else if(s==st_sn1){
				tkn0=parseInStatus_sn1(ch);
			}
			else if(s==st_sn2){
				tkn0=parseInStatus_sn2(ch);
			}
			else if(s==st_sn3){
				tkn0=parseInStatus_sn3(ch);
			}
			else if(s==st_cm1){
				tkn0=parseInStatus_cm1(ch);
			}
			else if(s==st_cm2){
				tkn0=parseInStatus_cm2(ch);
			}
			else if(s==st_rl1){
				tkn0=parseInStatus_rl1(ch);
			}
			else if(s==st_rl2){
				tkn0=parseInStatus_rl2(ch);
			}
			else if(s==st_var){
				tkn0=parseInStatus_var(ch);
			}
			else if(s==st_svr){
				tkn0=parseInStatus_svr(ch);
			}
			else{//s==bif
				tkn0=parseInStatus_bif(ch);
			}
			if(s!=st_end){
				ch=getChar();
			}
			if(tkn0!=null){
				tkn=tkn0;
			}
		}while(s!=st_end);
		s=st_nul;
		chs="";
		return tkn;
	}
	/**
	 * parse expression when expressionstaus is null
	 * @param curr
	 * @param rt
	 * @param od
	 * @param obj
	 * @return
	 * @throws ParserSyntaxException
	 * @throws ParserExpressionException
	 */
	private Expression parseExpr_null(Expression ex, ADQLToken obj)throws ParserSyntaxException,ParserExpressionException{
		switch(obj.getClassType()){
		case ADQLToken.tp_dbobject:			
			//break;
		case ADQLToken.tp_function:
			//break;
		case ADQLToken.tp_explicitfunction:
			//break;
		case ADQLToken.tp_tablecolumns:			
			ex.currod=obj;
			exprs=est_od;
			break;
		case ADQLToken.tp_operator:
			ADQLOperator op=(ADQLOperator)obj;
			if(op.getOpType()==op.op_plus || op.getOpType()==op.op_minus){
				
				ex.currod=new ADQLInteger();
				((ADQLInteger)(ex.currod)).setValue("0");
				ex.currop=op;
				ex.root=op;
				exprs=est_op1;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
		case ADQLToken.tp_keyword:
			//break;
		case ADQLToken.tp_comma:
			//break;
		case ADQLToken.tp_rparam:
			m_nLParam--;
			//break;
		case ADQLToken.tp_eof:
			exprs=est_end;
			break;			
		case ADQLToken.tp_star:
			exprs=est_sta;
			break;
		case ADQLToken.tp_integer:
			//break;
		case ADQLToken.tp_variable:
			//break;
		case ADQLToken.tp_sysvariable:
			//break;
		case ADQLToken.tp_real:
			ex.currod=obj;
			exprs=est_od;
			break;
		case ADQLToken.tp_lparam:
			m_nLParam++;
			ex=parseExpression();
			if(ex.terminator.getClassType()==ADQLToken.tp_rparam){
				//m_nLParam--;
				exprs=est_od;
			}
			else{//otherwise
				//updated by liuchao for covering an parsing error while 'where' condition has a lParam, 2008-03-31
				//ex.currod=new LParam();
				//exprs=est_end;
				exprs=est_end;
				//updated by liuchao for covering an parsing error while 'where' condition has a lParam, 2008-03-31
			}
			break;
		
		
		case ADQLToken.tp_comment:
			ex.currod=obj;
			exprs=est_end;
			break;
		default:
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_exprError);
			exprs=est_err;
		}
		return ex;
	}
	private Expression parseExpr_od(Expression ex,ADQLToken obj)throws  ParserSyntaxException, ParserExpressionException{

		switch(obj.getClassType()){
		case ADQLToken.tp_dbobject:
			//break;
		case ADQLToken.tp_function:
			//break;
		case ADQLToken.tp_explicitfunction:
			//break;
		case ADQLToken.tp_archivetable:
			//break;
		case ADQLToken.tp_builtinfunction:
			//break;
		case ADQLToken.tp_tablecolumns:
			{
				DASLog log=new DASLog();			
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
		case ADQLToken.tp_operator:
			if(ex.root!=null){
				ex=setop(ex,obj);				
			}
			else{
				ex.root=(ADQLOperator)obj;
				ex.currop=(ADQLOperator)obj;
				if(ex.currod!=null){
					ex.root.setLeft(ex.currod);
				}
			}
			exprs=est_op2;
			break;
		case ADQLToken.tp_rparam:
			m_nLParam--;
			exprs=est_end;
			break;
		case ADQLToken.tp_keyword:
		case ADQLToken.tp_comma:
			//break;
		
		case ADQLToken.tp_eof:
			exprs=est_end;			
			break;
			
		default:
			{
				DASLog log=new DASLog();				
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
		}
		return ex;
	}
	private Expression parseExpr_op1(Expression ex,ADQLToken obj)throws  ParserSyntaxException, ParserExpressionException{

		switch(obj.getClassType()){
		case ADQLToken.tp_dbobject:
			//break;
		case ADQLToken.tp_function:
			//break;
		case ADQLToken.tp_explicitfunction:
			//break;
		case ADQLToken.tp_tablecolumns:
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		case ADQLToken.tp_operator:
			//break;
		case ADQLToken.tp_keyword:
			//break;
		case ADQLToken.tp_comma:
			//break;
		case ADQLToken.tp_eof:
			//break;
		case ADQLToken.tp_star:
			//break;
		case ADQLToken.tp_rparam:
			//break;
		case ADQLToken.tp_comment:
			{
				DASLog log=new DASLog();			
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
		case ADQLToken.tp_integer:
			//break;
		case ADQLToken.tp_real:
			//break;
		case ADQLToken.tp_variable:
			//break;
		case ADQLToken.tp_sysvariable:	
			ex.currod=obj;
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		case ADQLToken.tp_archivetable:
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		case ADQLToken.tp_builtinfunction:
			break;
		case ADQLToken.tp_lparam:
			m_nLParam++;
			ex.currod=parseExpression().root;
			ex.currop.setRight(ex.currod);
			exprs=est_od;
			break;				
		default:
			{
				DASLog log=new DASLog();			
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
		}
		return ex;
	}
	private Expression parseExpr_op2(Expression ex,ADQLToken obj)throws  ParserSyntaxException, ParserExpressionException{

		switch(obj.getClassType()){
		case ADQLToken.tp_dbobject:
			//break;
		case ADQLToken.tp_function:
			//break;
		case ADQLToken.tp_explicitfunction:
			//break;
		case ADQLToken.tp_tablecolumns:
			ex.currod=obj;
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		case ADQLToken.tp_operator://modified by LC 20071018
			//<<<<<<<<<<<<<<<<<<<<added by LC 20071018
			ADQLOperator op=(ADQLOperator)obj;
			if(op.getOpType()==op.op_plus || op.getOpType()==op.op_minus){
				ADQLInteger od0=new ADQLInteger();
				od0.setValue("0");
				//ex.currop=op;
				//ex.root=op;
				//ex.currod=obj;
				ex.currop.setRight(od0);				
				ex=setop(ex,op);
				
				exprs=est_op2;
			}
			else{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
			//>>>>>>>>>>>>>>>>>>>>added by LC 20071018
			//break;
		case ADQLToken.tp_keyword:
			//break;
		case ADQLToken.tp_comma:
			//break;
		case ADQLToken.tp_star:
			//break;
		case ADQLToken.tp_comment:
			//break;
		case ADQLToken.tp_eof:
			{
				DASLog log=new DASLog();			
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
		case ADQLToken.tp_rparam:
			m_nLParam--;
			ex.currod=obj;
			ex.currop.setRight(obj);
			exprs=est_od;
			break;	
		case ADQLToken.tp_integer:
			//break;
		case ADQLToken.tp_real:
			//break;
			
		case ADQLToken.tp_variable:
			//break;
		case ADQLToken.tp_sysvariable:
			ex.currod=obj;
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		case ADQLToken.tp_lparam:
			ADQLParser p3=new ADQLParser();
			p3.m_strAdql=this.m_strAdql.substring(this.m_nOffset);
			m_nLParam++;
			ex.currod=(p3.parseExpression()).root;
			//this.m_nOffset+=p3.m_nOffset-1;//deleted by LC 20071018
			this.m_nOffset+=p3.m_nOffset;//added by LC 20071018
			ex.currop.setRight(ex.currod);
			exprs=est_od;
			break;
		case ADQLToken.tp_archivetable:
			//break;
		case ADQLToken.tp_builtinfunction:
			ex.currod=obj;
			ex.currop.setRight(obj);
			exprs=est_od;
			break;
		default:
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_exprError);
			exprs=est_err;
		}
		return ex;
	}
	private Expression parseExpr_sta(Expression ex,ADQLToken obj)throws  ParserSyntaxException, ParserExpressionException{
		
		switch(obj.getClassType()){
		case ADQLToken.tp_dbobject:
			//break;
		case ADQLToken.tp_function:
			//break;
		case ADQLToken.tp_explicitfunction:
			//break;
		case ADQLToken.tp_tablecolumns:
			//break;
		case ADQLToken.tp_operator:
			//break;
		case ADQLToken.tp_keyword:
			{
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			break;
		case ADQLToken.tp_rparam:
			m_nLParam--;
			exprs=est_end;
			break;
		case ADQLToken.tp_comma:
			//break;
		
			//break;
		case ADQLToken.tp_eof:
			exprs=est_end;
			break;
		
		default:
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_exprError);
			exprs=est_err;
		}
		return ex;
	}
	private Expression setop(Expression ex, ADQLToken obj)throws  ParserSyntaxException, ParserExpressionException{
		ADQLOperator opnew=(ADQLOperator)obj;
		while(true){
			if(ex.currop.getPriority()<opnew.getPriority()){
				opnew.setLeft(ex.currop.getRight());
				ex.currop.setRight(opnew);
				ex.currop=opnew;//added by LC 20071015
				break;
			}
			else if(ex.currop==ex.root){
				opnew.setLeft(ex.root);
				ex.root=opnew;
				ex.currop=ex.root;
				break;
			}
			else{
				ex.currop=ex.currop.getParent();
			}
		}		
		return ex;
	}
	/**
	 * parse expression generate expression tree
	 * @return
	 */
	private Expression parseExpression()throws ParserSyntaxException, ParserExpressionException{
		ADQLToken obj=parseToken();
		return parseExpression0(obj);
	}
	/**
	 * parse expression generate expression tree
	 * @return
	 */
	private Expression parseExpression0(ADQLToken obj)throws ParserSyntaxException, ParserExpressionException{
		ADQLOperator op_rt=null;
		ADQLOperator op_curr=null;		
		ADQLToken od=null;		
		Expression ex=new Expression();
		while(exprs!=est_end){
			switch(exprs){
			case est_nul:
				ex=parseExpr_null(ex,obj);
				//added by LC 2008-03-31
				if(exprs==est_end){
					obj=ex.terminator;
				}
				//added by LC 2008-03-31
				break;
			case est_od:
				ex=parseExpr_od(ex,obj);
				break;
			case est_op1:
				ex=parseExpr_op1(ex,obj);
				break;
			case est_op2:
				ex=parseExpr_op2(ex,obj);
				break;
			case est_sta:
				ex=parseExpr_sta(ex,obj);
				break;
			default:
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_exprError);
				exprs=est_err;
			}
			/*updated by LC 2008-03-31*/
			if(isDelimitor(obj)){
				exprs=est_end;
			}
			if(exprs!=est_end){
				obj=parseToken();
			}
			//updated by LC 2008-03-31
			/*if(exprs==est_end){
				obj=ex.terminator;
			}
			else{
				if(isDelimitor(obj)){
					exprs=est_end;
				}
				if(exprs!=est_end){
					obj=parseToken();
				}
			}*/
			//updated by LC 2008-03-31
		}	
		ex.terminator=obj;
		exprs=est_nul;
		return ex;
	}
	private boolean isDelimitor(ADQLToken obj){
		boolean dl=false;
		if(obj.getClassType()==ADQLToken.tp_comma ||
				obj.getClassType()==ADQLToken.tp_rparam||
				obj.getClassType()==ADQLToken.tp_keyword||//added by LC 2008-03-31
				obj.getClassType()==ADQLToken.tp_eof){
			dl=true;
		}
		return dl;
	}
	/**
	 * clear m_strDelimiter list
	 *
	 */
	private void clearDelimiter(){
		m_strDelimiter=null;		
	}
	/**
	 * add a token to delimiter selection
	 * @param deli: token which will be treated as a delimiter
	 */
	private void addDelimiter(ADQLToken deli){
		if(m_strDelimiter==null){
			m_strDelimiter=new LinkedList();
		}
		m_strDelimiter.add(deli);
	}
	/**
	 * check if the current token tkn is one of the delimiter tokens.
	 * @param tkn: current token
	 * @return true: it is a delimiter or false: it is not a delimiter
	 */
	private boolean checkDelimiter(ADQLToken tkn){
		boolean ret=false;
		for(int i=0;i<m_strDelimiter.size();i++){
			ADQLToken deli=(ADQLToken)m_strDelimiter.get(i);
			if(tkn.getADQLString().equals(deli.getADQLString())){
				ret=true;
				break;
			}
		}
		return ret;
	}
	private boolean isSelectTerminator(String k){
		if(k.equalsIgnoreCase("from") ||
				k.equalsIgnoreCase("where")||
				k.equalsIgnoreCase("group by")||
				k.equalsIgnoreCase("having")||
				k.equalsIgnoreCase("order by")){
			return true;
		}
		else 
			return false;
	}
	private boolean isFromTerminator(String k){
		if(k.equalsIgnoreCase("where")||
				k.equalsIgnoreCase("group by")||
				k.equalsIgnoreCase("having")||
				k.equalsIgnoreCase("order by")){
			return true;
		}
		else 
			return false;
	}
	private boolean isWhereTerminator(String k){
		if(	k.equalsIgnoreCase("group by")||
				k.equalsIgnoreCase("having")||
				k.equalsIgnoreCase("order by")){
			return true;
		}
		else 
			return false;
	}
/**
 *parse column items in select clause
 *@param tkn current ADQLToken object
 * @return first is select item object, second is token object
 */
	private Object[] parseColumn(ADQLToken currtkn)throws ParserSyntaxException,ParserExpressionException{
		ADQLToken tkn=null;
		SelectItem si=null;
		Expression expr=parseExpression0(currtkn);
		
		switch(expr.getRoot().getClassType()){
		case ADQLToken.tp_star:
			//break;
		case ADQLToken.tp_variable:
			//break;
		case ADQLToken.tp_tablecolumns:
			//break;
		case ADQLToken.tp_function:
			si=(SelectItem)tkn;
			break;
		case ADQLToken.tp_operator://an expression in the select clause
			
			//break;
		case ADQLToken.tp_dbobject:			
		    si=new Column((ADQLToken)(expr.getRoot()));
			tkn=expr.terminator;
			if(tkn.getValue().toString().equalsIgnoreCase("as")){
				tkn=parseToken();				
				if(tkn.getClassType()==ADQLToken.tp_dbobject){
					((Column)si).setAlias((DBObject)tkn);
					tkn=parseToken();
					if(tkn.getClassType()==ADQLToken.tp_comma || isSelectTerminator(tkn.getValue().toString())){
						expr.terminator=tkn;
					}
					else{
//						error
						DASLog log=new DASLog();
						log.error(ExceptionMessage.Par_syntaxError);
						log.debug("error location (ADQLParser.parseColumn)");
					}
				}				
				else{
					//error
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					log.debug("error location (ADQLParser.parseColumn)");
				}
			}
			else{
				if(tkn.getClassType()==ADQLToken.tp_dbobject){
					((Column)si).setAlias((DBObject)tkn);
					tkn=parseToken();
					if(tkn.getClassType()==ADQLToken.tp_comma || isSelectTerminator(tkn.getValue().toString())){
						expr.terminator=tkn;
					}
					else{
//						error
						DASLog log=new DASLog();
						log.error(ExceptionMessage.Par_syntaxError);
						log.debug("error location (ADQLParser.parseColumn)");
					}
				}
				else if(tkn.getClassType()==ADQLToken.tp_comma || isSelectTerminator(tkn.getValue().toString())){
					expr.terminator=tkn;
				}
				else{
					//error
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					log.debug("error location (ADQLParser.parseColumn)");
				}
			}
			break;
		default:
			if(expr.getRoot() ==null || isSelectTerminator(expr.getRoot().getValue().toString())){
				;
			}
			else{
				//error
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				log.debug("error location (ADQLParser.parseColumn)");
			}
		}
		Object[] ret={si,tkn};
		return ret;
	}
	private Object[] parseTableSource()throws ParserSyntaxException,ParserExpressionException{
		//ADQLToken tkn=null;
		Object[] ret={null,null};
		SubTableSource s=null;
		JoinedTable jt=null;
		//
			tkn=parseToken();
		
			
		if(tkn.getClassType()==ADQLToken.tp_archivetableitem){
			s=new SubTableSource();
			s.setArchiveTableItem((ArchiveTableItem)tkn);
			tkn=parseToken();
			if(tkn.getClassType()==tkn.tp_dbobject){
				s.setAlias((DBObject)tkn);
			}
			else{
				ret[1]=tkn;
			}
		}
		else if(tkn.getClassType()==ADQLToken.tp_keyword &&
				(tkn.getValue().toString().equalsIgnoreCase("inner") ||
						tkn.getValue().toString().equalsIgnoreCase("outer") ||
						tkn.getValue().toString().equalsIgnoreCase("left") ||
						tkn.getValue().toString().equalsIgnoreCase("right") ||
						tkn.getValue().toString().equalsIgnoreCase("full"))){
			ADQLToken tkn2=parseToken();
			if(tkn2.getClassType()==ADQLToken.tp_keyword && tkn2.getValue().toString().equalsIgnoreCase("join")){
				Object[] obj=parseTableSource();
				//
				if(obj[1]==null){
					tkn2=parseToken();
				}
				else{
					tkn2=(ADQLToken)obj[1];
				}
				if(tkn2.getClassType()==ADQLToken.tp_keyword && tkn2.getValue().toString().equalsIgnoreCase("on")){
					SearchCondition sc=parseCondition(null);
					jt=(JoinedTable)obj[0];
					if(tkn.getValue().toString().equalsIgnoreCase("inner")){
						jt.setJoinType(JoinedTable.JT_INNERJOIN);
					}
					else if(tkn.getValue().toString().equalsIgnoreCase("left") ){
						jt.setJoinType(JoinedTable.JT_LEFTJOIN);
					}
					else if(tkn.getValue().toString().equalsIgnoreCase("right")){
						jt.setJoinType(JoinedTable.JT_RIGHTJOIN);
					}
					else if(tkn.getValue().toString().equalsIgnoreCase("outer")){
						jt.setJoinType(JoinedTable.JT_OUTERJOIN);
					}
					else{
						jt.setJoinType(JoinedTable.JT_FULLJOIN);
					}
					jt.setOn(sc);					
				}
				else{
					//err
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					log.debug("error location (ADQLParser.parseTableSource)");
				}
			}
			else{
				//err
				DASLog log=new DASLog();
				log.error(ExceptionMessage.Par_syntaxError);
				log.debug("error location (ADQLParser.parseTableSource)");
			}
		}
		else if(tkn.getClassType()==ADQLToken.tp_keyword && tkn.getValue().toString().equalsIgnoreCase("cross")){
			ADQLToken tkn2=parseToken();
			if(tkn2.getClassType()==ADQLToken.tp_keyword &&tkn2.getValue().toString().equalsIgnoreCase("join")){
				Object[] obj=parseTableSource();
				jt=(JoinedTable)obj[0];
				jt.setJoinType(JoinedTable.JT_CROSSJOIN);
				ret[1]=obj[1];
			}
		}
		else{
			ret[1]=tkn;
		}
		if(s!=null){
			ret[0]=s;
		}
		else if(jt!=null){
			ret[0]= jt;
		}
		else{
			ret[0]=null;
		}
		return ret;
	}
	/**
	 * parse tree like condition structure in where clause
	 * @return
	 */
	private SearchCondition parseCondition(ADQLToken tkn)throws ParserSyntaxException, ParserExpressionException{
		SearchCondition scroot=null;
		SearchCondition sccurr=null;
		ADQLToken tk=null;
		if(tkn==null ||
				tkn.getValue().toString().equalsIgnoreCase("where")){
			tk=parseToken();
		}
		else{
			tk=tkn;
		}
		while(!isWhereTerminator(tk.getValue().toString()) &&
				tk.getClassType()!=ADQLToken.tp_eof){
			if(isKeyword(tk.getValue().toString())){ 
			   if(tk.getValue().toString().equalsIgnoreCase("AND")){
				   if(scroot!=null){
					   And sc0=new And();
					   SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
					   scroot=rsc[0];
					   sccurr=rsc[1];
				   }
				   else{
					   //error
					   DASLog log=new DASLog();			
						log.error(ExceptionMessage.SearchCondition_Error);
				   }
			   }
			   else if(tk.getValue().toString().equalsIgnoreCase("OR")){
				   if(scroot!=null){
					   Or sc0=new Or();
					   SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
					   scroot=rsc[0];
					   sccurr=rsc[1];
				   }
				   else{
					   //error
					   DASLog log=new DASLog();			
						log.error(ExceptionMessage.SearchCondition_Error);
				   }
			   }
			   else if(tk.getValue().toString().equalsIgnoreCase("NOT")){
				   Not sc0=null;
				   sc0=new Not();
				   if(scroot!=null){
					   
					   SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
					   scroot=rsc[0];
					   sccurr=rsc[1];
				   }
				   else{
					   scroot=sc0;
					   sccurr=sc0;
				   }
			   }
			   else{
				   DASLog log=new DASLog();
				   log.error(ExceptionMessage.Par_syntaxError);
				   log.debug("error location (ADQLParser.parseCondition)");
				   throw new ParserSyntaxException();
			   }
			   tk=parseToken();
			}
			else if(tk.getClassType()==ADQLToken.tp_explicitfunction && ((Function)tk).getName().equalsIgnoreCase("XMATCH")){
				XMatch sc0=new XMatch((Function)tk);
				SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
				scroot=rsc[0];
				sccurr=rsc[1];
				tk=parseToken();
			}
			else if(tk.getClassType()==ADQLToken.tp_explicitfunction && ((Function)tk).getName().equalsIgnoreCase("REGION")){
				Region sc0=new Region((Function)tk);
				SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
				scroot=rsc[0];
				sccurr=rsc[1];
				tk=parseToken();
			}
			else if(tk.getClassType()==ADQLToken.tp_dbobject ||
					tk.getClassType()==ADQLToken.tp_integer ||
					tk.getClassType()==ADQLToken.tp_real ||
					tk.getClassType()==ADQLToken.tp_explicitfunction){		
				Expression exp=parseExpression0(tk);
				CompareOp sc0=new CompareOp();
				sc0.setExpression(exp);
				SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
				scroot=rsc[0];
				sccurr=rsc[1];
				tk=exp.terminator;
			}
			else if(tk.getClassType()==ADQLToken.tp_lparam){
				//modified by LC 2008-03-31
				int flag1=m_nOffset;
				int currLParam=m_nLParam;
				//m_nLParam++;
				Expression exp=parseExpression0(tk);
				//SearchCondition sc0=parseCondition(null);//commented by LC  for test 2008-03-31
				//SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
				//scroot=rsc[0];
				//sccurr=rsc[1];
				if(m_nLParam==currLParam){/*param are closed*/
					CompareOp sc0=new CompareOp();
					sc0.setExpression(exp);
					SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
					scroot=rsc[0];
					sccurr=rsc[1];
					tk=exp.terminator;
				}
				else{ /*param is still open*/
					m_nOffset=flag1;
					SearchCondition sc0=parseCondition(null);
					SearchCondition[] rsc=SearchCondition.add(scroot, sccurr,sc0);
					scroot=rsc[0];
					sccurr=rsc[1];
					tk=parseToken();
				}
				
				
				//tk=parseToken();
//				modified by LC 2008-03-31
			}
			else if(tk.getClassType()==ADQLToken.tp_rparam){
				m_nLParam--;
				break;
			}
			
		}
		scroot.terminator=tk;
		return scroot;
	}
	/**
	 * not completed
	 * @return
	 * @throws ParserSyntaxException
	 */
	private SelectStatement parseSelectStatement()throws ParserSyntaxException,ParserExpressionException{
		ADQLToken tkn=null;
		SelectStatement ss=null;
		
		SelectClause sc=new SelectClause();
		try{
			tkn=parseToken();
		//parse select clause
		if(tkn.getClassType()==ADQLToken.tp_keyword && tkn.getValue().toString().equalsIgnoreCase("select")){
			tkn=parseToken();
			if(tkn.getClassType()==ADQLToken.tp_keyword && tkn.getValue().toString().equalsIgnoreCase("top") ){
				//parseTop();
				throw new ParserSyntaxException("TOP is not implemented");
			}		
			SelectList sl=new SelectList();			
			sc.setSelectList(sl);
			//sl.addSelectItem((SelectItem)tkn);
		
			while(!isSelectTerminator(tkn.getValue().toString()) ){
				Object[] d=parseColumn(tkn);
				sl.addSelectItem((SelectItem)d[0]);
				tkn=(ADQLToken)d[1];				
				if(tkn.getClassType()==ADQLToken.tp_comma){
					tkn=parseToken();
				}
				else if(isSelectTerminator(tkn.getValue().toString())){
					;
				}
				else{
//					err
					DASLog log=new DASLog();
					log.error(ExceptionMessage.Par_syntaxError);
					log.debug("error location (ADQLParser.parseSelectStatement)");
					throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
				}
			}
			
			sc.setSelectList(sl);
			ss=new SelectStatement();
			ss.setSelectClause(sc);				
			
		}
		else{
			//err
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			log.debug("error location (ADQLParser.parseSelectStatement)");
			throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
		}
		//parse from clause
		if(tkn.getClassType()==ADQLToken.tp_keyword && tkn.getValue().toString().equalsIgnoreCase("from")){
			//tkn=parseToken();
			From fm=new From();
			while(!isFromTerminator(tkn.getValue().toString()) 
					//&& tkn.getClassType()!=ADQLToken.tp_comma 
					&&
					tkn.getClassType()!=ADQLToken.tp_eof){
				Object[] obj=parseTableSource();
				if(obj[1]!=null){
					tkn=(ADQLToken)obj[1];
				}
				if(obj[0]!=null){
					if(obj[0].getClass().getName().equalsIgnoreCase("net.ivoa.adql.SubTableSource")){
						fm.addSubTableSource((SubTableSource)obj[0]);
						//tkn=parseToken();
					}
					else if(obj[0].getClass().getName().equalsIgnoreCase("net.ivoa.adql.JoinedTable")){
						fm.addJoinedTable((JoinedTable)obj[0]);
						//tkn=parseToken();
					}
					else{
						//err
						DASLog log=new DASLog();
						log.error(ExceptionMessage.Par_syntaxError);
						log.debug("error location (ADQLParser.parseSelectStatement)");
						throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
					}
				}
			}
			ss.setFrom(fm);
		}
		else{
			//err
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			log.debug("error location (ADQLParser.parseSelectStatement)");
			throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
		}
		//parse where clause
		if(tkn.getClassType()==ADQLToken.tp_keyword && 
				tkn.getValue().toString().equalsIgnoreCase("where") &&
				tkn.getClassType()!=ADQLToken.tp_eof){
			Where wh=new Where();
			
			if(!isWhereTerminator(tkn.getValue().toString()) &&
					tkn.getClassType()!=ADQLToken.tp_eof){
				SearchCondition s=parseCondition(tkn);
				wh.setSearchCondition(s);
				tkn=s.terminator;
			}
			ss.setWhere(wh);
		}
		else{
			if(tkn.getClassType()==ADQLToken.tp_eof){
				return ss;
			}
			//err
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			log.debug("error location (ADQLParser.parseSelectStatement)");
			throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
		}
		//parse having clause
		if(tkn.getClassType()==ADQLToken.tp_keyword && 
				tkn.getValue().toString().equalsIgnoreCase("having")){
			
		}
		else{
			if(tkn.getClassType()==ADQLToken.tp_eof){
				return ss;
			}
			//err
			DASLog log=new DASLog();
			log.error(ExceptionMessage.Par_syntaxError);
			log.debug("error location (ADQLParser.parseSelectStatement)");
			throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement)");
		}
	}catch(Exception e){
		throw new ParserSyntaxException("error location (ADQLParser.parseSelectStatement):"+e.getMessage());
	}
		return ss;
	}
	/**
	 * not completed
	 * @return
	 * @throws ParserSyntaxException
	 */
	private From parseFrom()throws ParserSyntaxException{
		return null;
	}
	/**
	 * parse an ADQL string and get an object tree which root is a SelectStatement object
	 * @param adql: ADQL string
	 * @return SelectStatement object contains ADQL object structure.
	 */
	public SelectStatement parse(String adql)throws ParserSyntaxException,ParserExpressionException{
		m_strAdql=adql;
		if(m_strAdql.charAt(m_strAdql.length()-1)!=';'){
			m_strAdql+=";";
		}
		SelectStatement ss=parseSelectStatement();
		
		return ss;
	}
	/**
	 * only for unit test
	 * @param args
	 */
	static public void main(String[] args){
		String str="select s.glon,s.glat,s.g from SDSS:Star2 s "+
		"where s.g>17.2 and (s.g<20.2 or s.g-s.r>=-0.3) and "+
		"s.g-s.r<=0 and s.u-s.g>=0.8 and s.u-s.g<=1.2 and (s.u-4*s.g+3*s.r<=1.8 "+
		"or s.u-2*s.g+s.r<=1.3 or s.u-14/15*s.g-1/15*s.r<=1.8 or s.u-2*s.g+s.r<=1.8)";
		//String str="select s.ra,s.decl from SDSS:Star2 s where s.g>16.3 AND (s.ra<19 or abs(s.dec-90)<(9+s.ra)*5)";
		//String str="select s.ra,s.decl from SDSS:Star2 s where s.g>16.3 AND (s.ra+15)<19 or abs(s.dec-90)<(9+s.ra)*5";
			
		//String str="select o.ra, o.decl from Archive:ppParams o where o.flag='nnnn' and o.d-8/o.feha>-9+10/o.a*8-o.dd and o.feha<-3+o.f/o.a*6/7-o.r";
		//String str="select o.ra as sss, o.decl from catalog:d o where  o.ra1-s.ra2<(3+s.dec3)/(o.a-o.b*8)-(4-5)*5";
		//String str="select o.ra as sss, o.decl from catalog:d o where  o.s='ttytd' and o.a>-8 and o.ra1-s.ra2<3+s.dec3";
		//String str="select o.ra as a,o.decl+s.dec+6, s.V from catalog:d o, sdss:ph s where o.ra-s.ra<3+s.dec and abs(o.ra)<1 and XMATCH(o,s,0.1)";// where o.ra>=3";
		//String str="select o.ra as ddd,o.decl  from catalog:d o, sdss:ph s,tmass:main t where o.ra1-s.ra2<3+s.dec3 and (not o.ra4) OR NOT s.sa5 AND abs(o.ra6)+sin(o.dec7+s.ra8)<0.2 AND REGION('circle J2000 120 23.7 0.2') OR XMATCH(o,s,0.1)";// where o.ra>=3";
		//String str="select o.ra as ddd,o.decl  from catalog:d o, sdss:ph s,tmass:main t where REGION('circle J2000 120 23.7 0.2') OR XMATCH(o,s,0.1)";// where o.ra>=3";
		System.out.println("origin:"+str);
		ADQLParser ap=new ADQLParser();
		ap.m_strAdql=str;
		if(ap.m_strAdql.charAt(ap.m_strAdql.length()-1)!=';'){
			ap.m_strAdql+=";";
		}
		ADQLToken tkn=null;
		try{/**
		do{
			
				tkn=ap.parseToken();
				System.out.println(tkn.getClassType()+":"+tkn.getValue().toString());
			
		}while(tkn!=null && tkn.getClassType()!=ADQLToken.tp_eof);/***/
		/**/
			SelectStatement ss=ap.parse(str);
			
			From fm=ss.getFrom();
			System.out.print("\nFrom ");
			Vector ts=fm.getTableSources();
			for(int i=0;i<ts.size();i++){
				SubTableSource sts=(SubTableSource)ts.get(i);
				System.out.println(sts.getArchiveTableItem().getADQLString());
			}
			//SubTableSource sts=(SubTableSource)ts.get(0);		
			//DBObject a=sts.getAlias();
			//System.out.println(sts.getArchiveTableItem().getADQLString());
			//System.out.println(sts.getArchiveTableItem().getTableName().getADQLString());
			//System.out.println(sts.getArchiveTableItem().getTableName().getFirst().getADQLString());
			//System.out.println("-->"+sts.getAlias().getADQLString());
			//sts=(SubTableSource)ts.get(1);		
			//DBObject a=sts.getAlias();
			//System.out.println(sts.getArchiveTableItem().getADQLString());
			//System.out.println(sts.getAlias().getADQLString());
			
			/*DBObject a=new DBObject();
			Identifier id=new Identifier();
			id.setValue("o");
			a.addFirst(id);
			SelectClause sc=ss.getSelectClause();
			SelectItem[] si=sc.getSelectItems();
			System.out.print("select ");
			Vector testDN=null;
			for(int i=0;i<si.length;i++){
				Column col=(Column)si[i];
				System.out.println(col.getADQLString());
				 testDN=new Vector();				
				testDN.add(a);
			    DBObject[] dblist=col.getAllDBObjectsWithoutCurrDN(testDN);
				int c=col.containsTables(testDN);
				//System.out.println("contains:"+c);
				if(dblist!=null){
					for(int j=0;j<dblist.length;j++){
						System.out.println("without:"+dblist[j].getADQLString());
						dblist[j].rename("m");
						System.out.println("rename:"+dblist[j].getADQLString());
					}
				}
				col.replaceName("o","NN");
				System.out.println("Replace:"+col.getADQLString());
			}*/
			Where wh=ss.getWhere();
			SearchCondition scd=wh.getSearchCondition();
			System.out.println("where "+scd.getADQLString());
			//System.out.println("contains "+a.getADQLString()+":"+scd.containsTables(testDN));
			//scd.replaceName("o","s");
			//System.out.println("replace ' result:"+scd.getADQLString());
			//System.out.println("contains "+a.getADQLString()+":"+scd.containsTables(testDN));
			Object[] scdobjs=scd.getAllDBObjects();
			System.out.println(scdobjs.length);
			/**/
		}
		catch(Exception e){
			System.out.println(e.getMessage());
		}
	}
}
